#!/opt/concourse/worker/volumes/live/95a891bf-867d-442e-50d6-9d61d0d17595/volume/anaconda-navigator_1619554653803/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/95a891bf-867d-442e-50d6-9d61d0d17595/volume/anaconda-navigator_1619554653803/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])
